@@ -1,4 +1,4 @@
-#' Randomizations
+#' Data-stream randomizations
 #'
 #' \code{randomizations} performs data-stream social network randomization. The function accepts a \code{data.table} with relocation data, individual identifiers and a randomization \code{type}. The \code{data.table} is randomized either using \code{step} or \code{daily} between-individual methods, or within-individual daily \code{trajectory} method described by Spiegel et al. (2016).
 #'
@@ -19,10 +19,12 @@
 #'   \item trajectory - datetime is \code{POSIXct} format
 #' }
 #'
-#' The \code{id}, \code{datetime},  (and optional \code{splitBy}) arguments expect the names of respective columns in \code{DT} which correspond to the individual identifier, date time, and additional grouping columns.
+#' The \code{id}, \code{datetime},  (and optional \code{splitBy}) arguments expect the names of respective columns in \code{DT} which correspond to the individual identifier, date time, and additional grouping columns. The \code{coords} argument is only required when the \code{type} is "trajectory", since the coordinates are required for recalculating spatial groups with \code{group_pts}, \code{group_lines} or \code{group_polys}.
 #'
 #' Please note that if the data extends over multiple years, a column indicating the year should be provided to the \code{splitBy} argument. This will ensure randomizations only occur within each year.
 #'
+#' The \code{group} argument is expected only when \code{type} is 'step' or 'daily'.
+#'
 #' For example, using \code{\link[data.table:year]{data.table::year}}:
 #'
 #' \preformatted{
@@ -30,11 +32,17 @@
 #' randomizations(DT, type = 'step', id = 'ID', datetime = 'timegroup', splitBy = 'yr')
 #' }
 #'
-#' The \code{iterations} is set to 1 if not provided. Take caution with a large value for \code{iterations} with large input \code{DT}.
+#' \code{iterations} is set to 1 if not provided. Take caution with a large value for \code{iterations} with large input \code{DT}.
 #'
-#' @return \code{randomizations} returns the random date time or random id along with the original \code{DT}, depending on the randomization \code{type}.
+#' @return \code{randomizations} returns the random date time or random id along with the original \code{DT}, depending on the randomization \code{type}. The length of the returned \code{data.table} is the original number of rows multiplied by the number of iterations + 1. For example, 3 iterations will return 4x - one observed and three randomized.
 #'
-#' In the case where \code{iterations = 1}:
+#' Two columns are always returned:
+#' \itemize{
+#'   \item observed - if the rows represent the observed (TRUE/FALSE)
+#'   \item iteration - iteration of rows (where 0 is the observed)
+#' }
+#'
+#' In addition, depending on the randomization type, random ID or random date time columns are returned:
 #'
 #' \itemize{
 #'   \item step - \code{randomID} each time step
@@ -42,19 +50,10 @@
 #'   \item trajectory - a random date time ("random" prefixed to \code{datetime} argument), observed \code{jul} and \code{randomJul} indicating the random day relocations are swapped to.
 #' }
 #'
-#' If \code{iterations > 1}, two more columns are returned.
-#' and return must be reassigned
-#'
-#' \itemize{
-#'   \item observed - if the rows represent the observed (TRUE/FALSE)
-#'   \item iteration - iteration of rows (where 0 is the observed)
-#' }
-#'
-#' A message is returned when any of these columns already exist in the input \code{DT}, because they will be overwritten.
-#'
 #'
 #' @param type one of 'daily', 'step' or 'trajectory' - see details
 #' @param datetime field used for providing date time or time group - see details
+#' @param group generated from spatial grouping functions - see details
 #' @inheritParams group_pts
 #' @param splitBy List of fields in DT to split the randomization process by
 #' @param iterations The number of iterations to randomize
@@ -63,6 +62,8 @@
 #'   \url{http://onlinelibrary.wiley.com/doi/10.1111/2041-210X.12553/full}
 #' @export
 #'
+#' @family Social network tools
+#'
 #' @examples
 #' # Load data.table
 #' library(data.table)
@@ -80,22 +81,12 @@
 #' # Spatial grouping with timegroup
 #' group_pts(DT, threshold = 5, id = 'ID', coords = c('X', 'Y'), timegroup = 'timegroup')
 #'
-#' ## Iterations = 1
-#' randomizations(
-#'     DT,
-#'     type = 'step',
-#'     id = 'ID',
-#'     datetime = 'timegroup',
-#'     iterations = 1,
-#'     splitBy = 'yr'
-#' )
-#'
-#' ## Iterations > 1
 #' # Randomization: step
 #' randStep <- randomizations(
 #'     DT,
 #'     type = 'step',
 #'     id = 'ID',
+#'     group = 'group',
 #'     datetime = 'timegroup',
 #'     splitBy = 'yr',
 #'     iterations = 2
@@ -106,6 +97,7 @@
 #'     DT,
 #'     type = 'daily',
 #'     id = 'ID',
+#'     group = 'group',
 #'     datetime = 'datetime',
 #'     splitBy = 'yr',
 #'     iterations = 2
@@ -116,6 +108,8 @@
 #'     DT,
 #'     type = 'trajectory',
 #'     id = 'ID',
+#'     group = NULL,
+#'     coords = c('X', 'Y'),
 #'     datetime = 'datetime',
 #'     splitBy = 'yr',
 #'     iterations = 2
@@ -124,6 +118,8 @@
 randomizations <- function(DT = NULL,
                            type = NULL,
                            id = NULL,
+                           group = NULL,
+                           coords = NULL,
                            datetime = NULL,
                            splitBy = NULL,
                            iterations = NULL) {
@@ -200,105 +196,69 @@ randomizations <- function(DT = NULL,
       )
     }
   }
+  if (type %in% c('step', 'daily')) {
+    if (is.null(group)) {
+      stop('group field must be provided if type is "step" or "daily"')
+    }
+    selCols <- c(splitBy, id, datetime, group)
+  } else if (type == 'trajectory') {
+    if (is.null(coords)) {
+      stop('coords must be provided if type is "trajectory"')
+    }
+    selCols <- c(splitBy, id, coords, datetime)
+  }
 
-  if (iterations == 1) {
-    if (type == 'step') {
-      if (is.null(splitBy)) {
-        splitBy <- datetime
-      } else {
-        splitBy <- c(datetime, splitBy)
-      }
-      if ('randomID' %in% colnames(DT)) {
-        message('column randomID found in DT, will be overwritten by this function')
-      }
-      DT[, randomID := .SD[sample(.N)], by = splitBy, .SDcols = id]
+  repDT <- DT[, .SD, .SDcols = selCols][rep(1:.N, iterations + 1)]
+  repDT[, rowID := .GRP, by = selCols]
 
-      return(DT[])
+  if (any(repDT[, .N , by = rowID]$N != iterations + 1)) {
+    warning('found none unique rows of id, datetime (and splitBy)')
+  }
 
+  repDT[, iteration := seq(0, .N - 1, 1), by = rowID]
+  repDT[iteration == 0, observed := TRUE]
+  repDT[iteration != 0, observed := FALSE]
+
+  set(repDT, j = 'rowID', value = NULL)
+
+  if (type == 'step') {
+    if (is.null(splitBy)) {
+      splitBy <- datetime
+    } else {
+      splitBy <- c(datetime, splitBy)
     }
 
-    if ('jul' %in% colnames(DT)) {
-      message('column jul found in DT, will be overwritten by this function')
-    }
+    repDT[!(observed), randomID := .SD[sample(.N)],
+          by = splitBy, .SDcols = id]
+    repDT[(observed), randomID := .SD, .SDcols = id]
+    return(repDT)
+  }
 
-    DT[, jul := data.table::yday(.SD[[1]]), .SDcols = datetime]
-    idDays <- unique(DT[, .SD, .SDcols = c(splitBy, id, 'jul')])
-    setnames(idDays, c(splitBy, id, 'jul'))
+  repDT[, jul := data.table::yday(.SD[[1]]), .SDcols = datetime]
+  idDays <- unique(repDT[, .SD,
+                         .SDcols = c(splitBy, id, 'jul',
+                                     'iteration', 'observed')])
 
-    if (type == 'daily') {
-      if ('randomID' %in% colnames(DT)) {
-        message('column randomID found in DT, will be overwritten by this function')
-      }
-      idDays[, randomID := .SD[sample(.N)],
-             by = c(splitBy, 'jul'), .SDcols = id]
+  if (type == 'daily') {
+    idDays[, randomID := .SD[sample(.N)],
+           by = c(splitBy, 'jul'), .SDcols = id]
+    idDays[(observed), randomID := .SD[[1]], .SDcols = id]
+    return(merge(repDT, idDays, on = c('iteration', 'jul', splitBy),
+                 all = TRUE))
 
-      return(merge(DT, idDays, on = c(splitBy, 'jul'))[])
-
-    } else if (type == 'trajectory') {
-      if ('randomJul' %in% colnames(DT)) {
-        message('column randomJul found in DT, will be overwritten by this function')
-      }
-      randomDateCol <- paste0('random', datetime)
-      if (randomDateCol %in% colnames(DT)) {
-        message(paste0('column ', randomDateCol,
-                       ' found in DT, will be overwritten by this function'))
-      }
-      idDays[, randomJul := sample(jul), by = c(id, splitBy)]
-      merged <- merge(DT, idDays, on = c('jul', id, splitBy))
-
-      merged[, (randomDateCol) :=
-               as.POSIXct(.SD[[1]] + (86400 * (randomJul - jul))),
-             .SDcols = datetime]
-      return(merged[])
-    }
-  } else {
-
-    selCols <- c(splitBy, id, datetime)
-    repDT <- DT[, .SD, .SDcols = selCols][rep(1:.N, iterations + 1)]
-    repDT[, rowID := .GRP, by = selCols]
-
-    if (any(repDT[, .N , by = rowID]$N != iterations + 1)) {
-      warning('found none unique rows of id, datetime (and splitBy)')
-    }
-
-    repDT[, iteration := seq(0, .N - 1, 1), by = rowID]
-    repDT[iteration == 0, observed := TRUE]
-    repDT[iteration != 0, observed := FALSE]
-
-    if (type == 'step') {
-      repDT[!(observed), randomID := .SD[sample(.N)],
-            by = splitBy, .SDcols = id]
-      repDT[(observed), randomID := .SD, .SDcols = id]
-      return(repDT)
-    }
-
-    repDT[, jul := data.table::yday(.SD[[1]]), .SDcols = datetime]
-    idDays <- unique(repDT[, .SD,
-                           .SDcols = c(splitBy, id, 'jul',
-                                       'iteration', 'observed')])
-    # setnames(idDays, c(splitBy, id, 'jul', 'iteration', 'observed'))
-
-    if (type == 'daily') {
-      idDays[, randomID := .SD[sample(.N)],
-             by = c(splitBy, 'jul'), .SDcols = id]
-      idDays[(observed), randomID := .SD[[1]], .SDcols = id]
-      return(merge(repDT, idDays, on = c('iteration', 'jul', splitBy),
-                   all = TRUE))
-
-    } else if(type == 'trajectory'){
-      idDays[, randomJul := sample(jul),
-             by = c(id, splitBy, 'iteration')]
-      merged <- merge(repDT, idDays,
-                      on = c('jul', id, 'iteration', splitBy),
-                      all = TRUE)
-      randomDateCol <- paste0('random', datetime)
-      merged[, (randomDateCol) :=
-               as.POSIXct(.SD[[1]] + (86400 * (randomJul - jul))),
-             .SDcols = datetime]
-      merged[(observed),
-             c(randomDateCol, 'randomJul') := .SD,
-             .SDcols = c(datetime, 'jul')]
-      return(merged)
-    }
+  } else if (type == 'trajectory') {
+    idDays[, randomJul := sample(jul),
+           by = c(id, splitBy, 'iteration')]
+    merged <- merge(repDT, idDays,
+                    on = c('jul', id, 'iteration', splitBy),
+                    all = TRUE)
+    randomDateCol <- paste0('random', datetime)
+    merged[, (randomDateCol) :=
+             as.POSIXct(.SD[[1]] + (86400 * (randomJul - jul))),
+           .SDcols = datetime]
+    merged[(observed),
+           c(randomDateCol, 'randomJul') := .SD,
+           .SDcols = c(datetime, 'jul')]
+    return(merged)
   }
 }
